<?php

/*
 * Created on Aug 6, 2007
 *
 * To change the template for this generated file go to
 * Window - Preferences - PHPeclipse - PHP - Code Templates
 */
 
require_once('midgard_admin_sitewizard_creator.php');
require_once('midgard_admin_sitewizard_creator_structure.php');
require_once('midgard_admin_sitewizard_creator_sitegroup.php');
require_once('midgard_admin_sitewizard_exception.php');

class midgard_admin_sitewizard_creator_host extends midgard_admin_sitewizard_creator
{
    private $sitegroup = null;
    private $host = null;
    private $root_page = null;
    private $host_url = '';
    private $host_prefix = '';
    private $host_style_id = null;

    private $root_topic;
    
    private $objects_array = array();
    
    public function __construct(midgard_setup_config &$config, $parent_link = null)
    {
        parent::__construct(&$config, $parent_link);
        
        if (is_object($parent_link))
        {
            $this->sitegroup = $this->parent_link->get_sitegroup();
        }
    }
    
    public function initialize($sitegroup_guid)
    {
        $this->verbose('Initializing host creation');
    
        if (is_object($this->sitegroup))
        {        
            if (!$this->sitegroup = mgd_get_sitegroup($sitegroup_guid) /*new midgard_sitegroup()*/)
            {
                throw new midgard_admin_sitewizard_exception("Sitewizard couldn't initialize sitegroup object.
                    Reason: " . mgd_errstr());
            }
            else
            {
                $this->verbose("Getting sitegroup for host creation GUID: " . $this->sitegroup->guid);
                
                return true;
            }
        }
        
        return true;
    }
    
    public function set_sitegroup(MidgardSitegroup $sitegroup)
    {
        $this->sitegroup = $sitegroup;
    }

    private function create_host()
    {
        $this->verbose("Starting to create host for sitegroup GUID: " . $this->sitegroup->guid);
    
        $this->host = new midgard_host();
        $this->host->sitegroup = $this->sitegroup->id;
        $this->host->name = $this->host_url;
        $this->host->prefix = $this->host_prefix;
        $this->host->port = 80;
        $this->host->online = 1;
        $this->host->owner = $this->sitegroup->admingroup;
        
        if (!is_null($this->host_style_id))
        {
            $this->host->style = $this->host_style_id;
        }
        
        if (!$this->host->create())
        {
            throw new midgard_admin_sitewizard_exception("Failed to create host for sitegroup GUID: " 
                . $this->sitegroup->guid);
        }
        else
        {
            $this->verbose("Created host PREFIX: " . $this->host->name . $this->host->prefix . " GUID: " 
                . $this->host->guid);
        
            return true;
        }
    }

    private function create_root_topic()
    {
        $_topic = new midgard_topic();
        $_topic->sitegroup = $this->root_page->sitegroup;
        $_topic->name = $this->host->name . $this->host->prefix . " root topic";
        $_topic->extra = $this->host->name . $this->host->prefix;
        $_topic->style = "inherited";
        $_topic->component = "net_nehmer_static";

        $created = $_topic->create();
        
        if(!$created)
        {
            $this->setup_config->ui->warning(_("Failed to create root topic"));
            return FALSE;
        }

        $param_created = 
            $this->host->parameter("midgard", "midcom_root_topic_guid", $_topic->guid);

        if(!$param_created)
        {
            $this->setup_config->ui->warning(_("Failed to configure host topic"));
            return FALSE;
        }

        $this->root_topic = &$_topic;

        return TRUE;
    }

    private function create_code_init()
    {
        /* TODO, add log message */
        $element = new midgard_pageelement();
        $element->sitegroup = $this->root_page->sitegroup;
        $element->name = "code-init";
        $element->page = $this->root_page->id;
        $guid = $this->root_topic->guid;
        $element->value = "
<(code-init-before-midcom)>
<?php
\$GLOBALS['midcom_config_local']['midcom_root_topic_guid'] = \"$guid\";

require 'midcom/lib/midcom.php';
?><(code-init-after-midcom)><?php
\$_MIDCOM->codeinit();

?>";

        $created = $element->create();

        if(!$created) {
            
            $this->setup_config->ui->warning(_("Failed to create code-init"));
            return FALSE;
        }

        return TRUE;
    }

    private function create_root_page()
    {
        $this->verbose("Starting to create root page for sitegroup GUID: " . $this->sitegroup->guid);    
    
        $this->root_page = new midgard_page();
        $this->root_page->sitegroup = $this->sitegroup->id;
        $this->root_page->up = 0;
        $this->root_page->name = $this->host->name . $this->host->prefix . " root";
        $this->root_page->title = $this->sitegroup->name;

        /* FIXME , it MUST be moved to boolean in core! */
        $this->root_page->info = 'active';

        /* content */
        $this->root_page->content = "<?php echo \$_MIDCOM->content(); ?>";
        
        if (!$this->root_page->create())
        {
            throw new midgard_admin_sitewizard_exception("Failed to create root page for sitegroup GUID " 
                . $this->sitegroup->guid);
        }     
        else
        {
            $this->verbose("Created root page for sitegroup GUID: " . $this->sitegroup->guid);
        
            $this->host->root = $this->root_page->id;
            $this->create_root_topic();
            $this->create_code_init();

            if (!$this->host->update())
            {
                throw new midgard_admin_sitewizard_exception("Failed to update hosts root property. Reason: " 
                    . mgd_errstr());
            }
            else
            {
                $this->verbose("Updated new hosts root property.");
            
                return true;
            }
        }
    }
   
    /** 
     * @param string style_name, name of the style
     * @return TRUE on success, FALSE otherwise
     *
     * This function returns false on failure and lets application 
     * ( like wizard setup ) to allow ask user again.
     * Basically, user may type or select wrong syle name, so setup
     * should not fail here , but instead fallback to previous state.
     */
    public function set_host_style($style_name)
    {
        /* I am not sure if we want to check sitegroup context here.
         * If there's no need to create SG0 hosts we should check 
         * if we are in SGX and throw an error when SG0 is used 
         */
        
        /* PHP fundamental types in function arguments "rocks" :( */
        if(!is_string($style_name))
        {
            $unexpected_type = gettype($style_name);
            $this->setup_config->ui->warning(_("Can not set host's style. Unexpected style name type") . " ( {$unexpected_type} )");

            return FALSE;
        }

        $qb = new midgard_query_builder("midgard_style");
        $qb->add_constraint("name", "=", $style_name);
    
        $ret = $qb->execute();

        if(count($ret) == 1) 
        {
            $this->host_style_id = $ret[0]->id;
        
        } else {
            
            $this->setup_config->ui->warning(_("Can not find style") . " ( {$style_name} )");
            return FALSE;
        }
    }
    
    public function set_host_url($host_url)
    {
        $this->verbose("Setting host url \"" . $host_url . "\"");
    
        $this->host_url = $host_url;
    }
    
    public function set_host_prefix($host_prefix)
    {
        $this->verbose("Setting host prefix \"" . $host_prefix . "\"");

        $this->host_prefix = $host_prefix;
    }
    
    public function cleanup()
    {
    
    }
    
    public function next_link()
    {
        $this->verbose('Initializing next link in the creation chain.');
        $this->next_link = new midgard_admin_sitewizard_creator_structure($this);
        
        return $this->next_link;
    }
    
    public function previous_link()
    {
        $this->verbose('Returning previous link in the creation chain.');
    
        return $this->parent_link;
    }
    
    public function execute()
    {
        $this->verbose('Starting execution chain (midgard_admin_sitewizard_creator_host)');
        
        if ($this->parent_link != null)
        {
            $this->verbose('Executing parent');
            
            try
            {
                $this->parent_link->execute();		
            }
            catch (midgard_admin_sitewizard_exception $e)
            {
                $e->error();
                throw new midgard_admin_sitewizard_exception("Failed to execute parent creator");
            }
        }

        try 
        {
            $this->create_host();
            $this->create_root_page();
            $this->verbose("Sitewizard created host successfully. Now cleaning up...");
            $this->cleanup();
            //return $this->creation_root_topic->guid;
        }
        catch(midgard_admin_sitewizard_exception $e)
        {
            $e->error();      
            $this->cleanup();
            throw new midgard_admin_sitewizard_exception("Failed to create host");
            return false;
        }    
    }


}

?>
