<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 softtabstop=4: */

require_once 'Console/Getargs.php';
require_once 'midgard_quick_setup.php';

$opts_config =array();

/* Setup type. Quick or interactive one */
$opts_config['type'] = array (
    'short' => 't',
    'max'   => 1,
    'min'   => 1,
    'desc'  => 'The type of installation.',
    'default' => 'quick',
);
$setup_types = array('wizard', 'quick', 'config');

/* Action to perform. Install, update, install pear packages, etc */
$opts_config['action'] = array (
    'short' => 'a',
    'max'   => 1,
    'min'   => 1,
    'desc'  => 'Setup action to perform (install by default).',
    'default' => 'install',
);
$setup_actions = array('install', 'update', 'pear', 'vhost', 'dbupdate', 'config-set');   

/* Configuration name , by default we should keep config file named from database name */
$opts_config['configuration'] = array (
    'short' => 'c',
    'max'   => 1,
    'min'   => 0,
    'desc'  => 'Midgard configuration name.',
    'default' => ""
);

/* Set configuration key's value */
$opts_config['config-set'] = array (
    'short' => 's',
    'max'   => 2,
    'min'   => 2,
    'desc'  => 'Sets key value for the named configuration.',
    'default' => ""
);

/* Set verbose level*/
$opts_config['verbose'] = array (
    'short' => 'v',
    'max'   => 1,
    'min'   => 1,
    'desc'  => 'Sets verbose level.',
    'default' => 1
);

$args = Console_Getargs::factory($opts_config);
if (PEAR::isError($args))
{
    midgard_setup_ui_cli::error($args->getMessage());
}

/* Check if user's values are allowed.
 I have no idea if Console_Getargs supports such feature */
$arg_type = $args->getValue('type');
if(!in_array($arg_type, $setup_types))
    midgard_setup_ui_cli::error("'{$arg_type}'" . _(" is not supported type of installation"));

$arg_action = $args->getValue('action');
if(!in_array($arg_action, $setup_actions))
    midgard_setup_ui_cli::error("'{$arg_action}'" . _(" is not supported type of actions"));

$arg_config = $args->getValue('configuration');

$arg_verbose = $args->getValue('verbose');

if($arg_verbose > 0)
    midgard_setup_ui_cli::message(_("Starting Midgard setup"));

if($arg_verbose > 0)
    midgard_setup_ui_cli::message("'{$arg_type}'" . _(" type selected"));

/* Initialize setup */
switch ($arg_type) {

    case 'wizard':
       
       midgard_setup_ui_cli::error("Wizard type not yet implemented");
       
       break;

    case 'config':
        
        /* Be quiet */
        
        break;

    default:
        
        try {
            
            $setup = new midgard_quick_setup($arg_config);

        } catch (Exception $e) {
            
            midgard_setup_ui_cli::error($e->getMessage());
        }
        
        break;
}

switch ($arg_action) {

    case 'install':
        
        /* This is temporary hack.
         * It should be removed once role_midgardelement gets username using midgard_config class 
         */
        //$config = &$setup->midgard_config;
        $config = new midgard_config();
        $config->read_file("midgard");
        $config->midgardusername = "admin";
        $config->midgardpassword = "password";
        $config->save_file("midgard");

        $setup->install_database();
        $setup->configure_pear();
        $setup->install_pear();
        
        break;

    case 'dbupdate':
        
        $setup->update_database();
        
        break;
    
    case 'vhost':      
        
        $setup->configure_sitegroup();
        $setup->install_sitegroup();
        $setup->configure_vhost();
        $setup->install_vhost();
        
        break;

    case 'config-set':

        if($arg_config == "")
            midgard_setup_ui_cli::error(_("'config-set' requires configuration"));

        $mc = new midgard_config();
        $mc->read_file($arg_config);
       
        $kv = $args->getValue('config-set');

        $mc->{$kv[0]} = $kv[1];
        $mc->save_file($arg_config);
        
        break;

    /* should it happen? probably there must be monty in actions array */
    default:
        
        midgard_setup_ui_cli::monty();
        
        break;
}

?>
