<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 softtabstop=4 */

define("MIDCOM_STATIC_DIR_REL", "midcom/static");

require_once 'PEAR.php';
require_once 'PEAR/Config.php';
require_once 'PEAR/Command/Channels.php';
require_once 'PEAR/Command/Config.php';
require_once 'PEAR/Frontend/CLI.php';
require_once 'PEAR/Installer.php';
require_once 'PEAR/Installer/Role.php';
require_once 'PEAR/Registry.php';
require_once 'PEAR/Downloader/Package.php';
require_once 'midgard_setup_globals.php';
require_once 'midgard_setup_log.php';

class midgard_setup_pear {

    private $midcom_channel;
    private $midcom_channel_alias;
    private $midcom_channel_url;
    private $midcom_static_dir;
    private $midcom_channel_base;
    private $pearified_channel;
    private $pearified_channel_url;
    private $pearified_channel_alias;
    private $initial_base_packages;
    private $initial_midgard_packages = array();
    private $initial_midgard_templates = array();
    private $initial_pear_dir;
    private $initial_web_dir;
    private $pear_dir;
    private $web_dir;

    private $pear_ui;
    private $pear_config;
    private $pear_registery;
    private $pear_channel;
    private $pear_installer;
    private $pear_downloader;

    public function __construct()
    {
        midgard_setup_log::write(_("Initialize new ").__CLASS__);

        $this->midcom_channel = "pear.midcom-project.org";
        $this->midcom_channel_alias = "midcom";
        $this->midcom_channel_base = 
            "http://pear.midcom-project.org/Chiara_PEAR_Server_REST";

        $this->pearified_channel = "pearified.com";
        $this->pearified_channel_alias = "pearified";
        $this->pearified_channel_base =
            "http://pearified.com/Chiara_PEAR_Server_REST";
        		
        $this->initial_base_packages = 
            array (
                "HTML_QuickForm",
                "HTML_Common"
                );
        
        /* Read simple txt files and create arrays for initial packages and templates */
        self::get_packages_list(&$this->initial_midgard_packages, "midgard_setup_pear_packages");
        self::get_packages_list(&$this->initial_midgard_templates, "midgard_setup_pear_templates");

        $this->pear_config = PEAR_Config::singleton();
        if(PEAR::IsError($this->pear_config))
        {
            $msg = "Failed to initialize PEAR_Config::singleton";
            midgard_setup_log::write_exception($msg);
            throw new Exception($msg, 0);
        }
        
        $this->set_state("beta");

        $this->initial_php_dir = $this->pear_config->get('php_dir');   
        
        $this->set_midcom_static();
        
        $this->pear_ui = new PEAR_Frontend_CLI(); 
        if(PEAR::IsError($this->pear_ui))
        {
            $msg = "Failed to initialize PEAR_Frontend_CLI";
            midgard_setup_log::write_exception($msg);
            throw new Exception($msg, 0);
        }

        $this->pear_channel = 
            new PEAR_Command_Channels(&$this->pear_ui, &$this->pear_config);
         
        if(PEAR::IsError($this->pear_channel))
        {
            $msg = "Failed to initialize PEAR_Command_Channels";
            midgard_setup_log::write_exception($msg);
            throw new Exception($msg, 0);          
        }

        $this->pear_installer = new PEAR_Installer(&$this->pear_ui, $this->pear_ui);
        if(PEAR::IsError($this->pear_installer))
        {
            $msg = "Failed to initialize PEAR_Installer";
            midgard_setup_log::write_exception($msg);
            throw new Exception($msg, 0);                     
        }
                
        $this->pear_downloader = new PEAR_Downloader(&$this->pear_ui, 
            array('force' => 'yes'), &$this->pear_config);
        if(PEAR::IsError($this->pear_downloader))
        {
            $msg = "Failed to initialize PEAR_Downloader";
            midgard_setup_log::write_exception($msg);
            throw new Exception($msg, 0);                     
        }

        $this->pear_registry = new PEAR_Registry();
        if(PEAR::IsError($this->pear_registry))
        {
            $msg = "Failed to initialize PEAR_registry";
            midgard_setup_log::write_exception($msg);
            throw new Exception($msg, 0);                     
        }        
    }

    private function set_midcom_static()
    {
        if(!$this->pear_config->isDefined("web_dir"))
            return;

        $this->initial_web_dir = $this->pear_config->get('web_dir');
        
        if(!strstr($this->initial_web_dir, 'midcom'))
        {
            $this->midcom_static_dir = 
                $this->initial_web_dir."/".MIDCOM_STATIC_DIR_REL;
                
        } else {
        
            $this->midcom_static_dir = $this->initial_web_dir;
        }

        $set = $this->pear_config->set('web_dir', $this->midcom_static_dir, 'user');
            
        if(!$set) 
            midgard_setup_ui_cli::warning(_("Failed to set '{$this->midcom_static_dir}' web_dir"));
            
        /* write configuration */
        $this->pear_config->writeConfigFile();
    }

    private function get_packages_list(array &$packages_array, $filename)
    {
    	$filepath = MIDGARD_SETUP_DIRECTORY_USR."/share/midgard/setup/php/".$filename;

        if(!$content = file($filepath))
            midgard_setup_ui_cli::warning(_("Failed to get packages list from file") . "( {$filepath} )");

        foreach($content as $line)
        {
            $line = trim($line);

            /* TODO, add more validation if needed */
            if(strstr($line, "#") 
                || $line == "")
                continue;
            
            $packages_array[] = $line;
        }
    }

    final public function midcom_static_dir()
    {
        return $this->midcom_static_dir;
    }

    public function set_state($state = 'beta')
    {
        /* TODO, I have no idea if it's pear issue.
         * It doesn't work when state is set per channel, so we have to 
         * set state for all channels ( default false ).
         */
        $_channel = $this->midcom_channel;
        $_channel = false;

        $set = $this->pear_config->set('preferred_state', 
            $state, 'system', $_channel);

        if(!$set)
            midgard_setup_ui_cli::warning(_("Failed to set '{$state}' prefrred_state"));
        else 
            midgard_setup_ui_cli::message(_("Preferred pear package state changed") . " ( {$state} )");
    }

    /* This method actually reloads config object associated with our class
    instance. But the point is to get role's config keys available just after
    packages has been installed. Keep in mind that running pear from command 
    line you execute separate processes with completely new ( refreshed ) 
    instances. During any midgard_setup_pear execution we need to invoke this
    method to "workaround" not existing config keys issue */
    private static function reload_roles(&$config)
    {
        PEAR_Installer_Role::registerRoles();
        PEAR_Installer_Role::initializeConfig(&$config);
    }

    final public function install_midcom_roles()
    { 
        midgard_setup_log::write(_("Installing MidCOM roles"));
        
        /* Install or upgrade Role_Web */
        $this->install_pearified_base_package("Role_Web");
        midgard_setup_pear::reload_roles(&$this->pear_config);

        $this->set_midcom_static();	

        $this->install_midcom_package('Role_Mgdschema', TRUE); 
        midgard_setup_pear::reload_roles(&$this->pear_config);

        $mgdschema_dir = MIDGARD_SETUP_MIDGARD_PREFIX . "/share/schema";
        $set = $this->pear_config->set('mgdschema_dir',  
            $mgdschema_dir, 'system');
    
        if(!$set)
            midgard_setup_ui_cli::warning(_("Failed to set '{$mgdschema_dir}' mgdschema_dir"));

        $this->install_midcom_package('Role_Midgardsql', TRUE);
        midgard_setup_pear::reload_roles(&$this->pear_config);

        $midgardsql_dir = MIDGARD_SETUP_MIDGARD_PREFIX . "/share/sql/update";
        $set = $this->pear_config->set('midgardsql_dir', 
            $midgardsql_dir, 'system');
    
        if(!$set)
            midgard_setup_ui_cli::warning(_("Failed to set '{$midgardsql_dir}' midgardsql_dir"));

        $this->install_midcom_package('Role_Midgardelement', TRUE);
        midgard_setup_pear::reload_roles(&$this->pear_config);

        $mgdschema_dir = MIDGARD_SETUP_MIDGARD_PREFIX . "/share/schema";
        midgard_setup_ui_cli::warning("midgard_config_file not set");
        /*
        $set = $this->pear_config->set('mgdschema_dir', 
            $state, 'system', $mgdschema_dir);
    
        if(!$set)
            midgard_setup_ui_cli::warning(_("Failed to set '{$mgdschema_dir}' mgdschema_dir")); */

    }

    final public function install_base_packages()
    {      
        midgard_setup_ui_cli::message(_("Installing MidCOM base packages"));

        foreach($this->initial_midgard_packages as $name) 
        {
            $this->install_midcom_package($name);        
        }

        /* We force templates installation.
         * Package might be already installed but is not imported to database.
         * Package itself will take action itself if it's already installed or not */
       	midgard_setup_ui_cli::message(_("Installing MidCOM base templates"));

        foreach($this->initial_midgard_templates as $name) 
        {
            $this->install_midcom_package($name, true);        
        }
    }

    private function install_pearified_base_package($name, $force = FALSE)
    {
        midgard_setup_log::write(_("Installing base packages using pearified.com channel"));

        $pkg_name = $this->pearified_channel_alias . "/" . $name;

        $options = array('onlyreqdeps' => 'true', 'upgrade' => 'true');

        if($force)
            $options['force'] = 'true';

        if($this->package_is_newer($name, $this->pearified_channel, 
            $this->pearified_channel_alias, $this->pearified_channel_base))
        {
            midgard_setup_ui_cli::message(_("Installing $name"));
            
            $downld = $this->pear_downloader->download(array($pkg_name));
            
            if(PEAR::isError($downld))
                midgard_setup_ui_cli::error("PEAR Downloader: ".$downld->getMessage());

            $installed = $this->pear_installer->install($downld[0], $options);
            
            if(PEAR::isError($installed))
                midgard_setup_ui_cli::error("PEAR Installer: ".$installed->getMessage());

            return TRUE;
        
        } else {
        
            midgard_setup_ui_cli::message(_("$name already installed"));
            return TRUE;
        }
        
        return FALSE;
    }

    final public function install_midcom_package($name, $force = FALSE)
    {
        midgard_setup_log::write(_("Installing MidCOM package ").$name);

        $pkg_name = $this->midcom_channel_alias . "/" . $name;

        $options = array('onlyreqdeps' => 'true', 'upgrade' => 'true');

        if($force) 
        {
            $options['force'] = 'true';
            
            $downld = $this->pear_downloader->download(array($pkg_name));
            
            if(PEAR::isError($downld))
                midgard_setup_ui_cli::error("PEAR Downloader: ".$downld->getMessage());
                
            $installed = $this->pear_installer->install($downld[0], $options);
            
            if(PEAR::isError($installed))
                midgard_setup_ui_cli::error("PEAR Installer: ".$installed->getMessage());
                
            return TRUE;
        }

        /* We should check if package exists in channel or newly updated 
        channel xml file. Registry check , checks only local data.
        if($this->pear_registry->packageExists($name, $this->midcom_channel))
        {
            midgard_setup_ui_cli::warning(_("Can not install $pkg_name package. It doesn't exists in $this->midcom_channel channel"));
        }
        */

        if($this->package_is_newer($name, $this->midcom_channel, 
            $this->midcom_channel_alias, $this->midcom_channel_base))
        {          
            $downld = $this->pear_downloader->download(array($pkg_name));
            
            if(PEAR::isError($downld))
                midgard_setup_ui_cli::error("PEAR Downloader: ".$downld->getMessage());

            $installed = $this->pear_installer->install($downld[0], $options);
            
            if(PEAR::isError($installed))
                midgard_setup_ui_cli::error("PEAR Installer: ".$installed->getMessage());

            return TRUE;
        
        } else {
        
            midgard_setup_ui_cli::message(_("$name already installed"));
            return TRUE;
        }
        
        return FALSE;
    }

    private function package_is_newer($name, $channel, $channel_alias, $channel_base)
    {
        $installed_version = 
            $this->pear_registry->packageInfo($name, 'version', $channel);
        
        if(is_null($installed_version)) 
            return TRUE;

        $rest = $this->pear_config->getREST("1.0");

        $params = array (
            'package' => $name,
            'channel' => $channel_alias
            );

        $state = $this->pear_config->get('preferred_state');

        $url = $rest->getDownloadURL($channel_base . "/", 
            $params, $state, $installed_version);

        if(PEAR::isError($url)) 
        {
            midgard_setup_ui_cli::error($url->getMessage());
        }

        /* 
        $txt = $name . " - installed: ". $installed_version ." available: ". $url['version'];
        midgard_setup_ui_cli::message($txt);
        */

        if (version_compare($installed_version, $url['version'], '>=')) 
        { 
            return FALSE;
        }

        return TRUE;
    }

    final public function set_channels()
    {
        midgard_setup_log::write(_("Setting MidCOM channels"));

        /* update pear.php.net */
        $ret = $this->pear_channel->doUpdate('channel-update', 
            array(), array('pear.php.net')); 

        if (PEAR::isError($ret)) 
        {        
            midgard_setup_ui_cli::error($ret->getMessage());
        }
        
        /* discover pear.midcom-project.org */
        if(!$this->pear_registry->channelExists($this->midcom_channel)) 
        {
       
            $ret = $this->pear_channel->doDiscover('channel-add', 
                array(), array($this->midcom_channel));
            
            if(PEAR::isError($ret))
                midgard_setup_ui_cli::error($ret->getMessage());
        }

        /* discover pearfified.com */
        if(!$this->pear_registry->channelExists($this->pearified_channel)) 
        {
            $ret = $this->pear_channel->doDiscover('channel-add', 
                array(), array($this->pearified_channel));
            
            if(PEAR::isError($ret))
                midgard_setup_ui_cli::error($ret->getMessage());      
        }
    }

    final public function update_channels()
    {
        midgard_setup_log::write(_("Updating MidCOM channels"));

        /* update midcom channel */
        $ret = $this->pear_channel->doUpdate('channel-update', 
            array(), array($this->midcom_channel));
        
        if (PEAR::isError($ret)) 
            midgard_setup_ui_cli::error($ret->getMessage());

        /* update pearfied channel */
        $ret = $this->pear_channel->doUpdate('channel-update', 
            array(), array($this->pearified_channel));
        
        if (PEAR::isError($ret)) 
            midgard_setup_ui_cli::error($ret->getMessage());
    }
        
    /* TODO */
    /* Add destructor which reverts web_dir and php_dir back.
    We should define own dirs per instance life cycle */
}

?>
