<?php

require_once 'midgard_setup_ui.php';

class midgard_setup_cli_exception extends Exception 
{
    public function __construct($message, $code = 0)
    {
        parent::__construct($message, $code);
    }

    public function message()
    {
        $msg = "{$this->message} ( {$this->file}:{$this->line} )";
        midgard_setup_log::write_exception($msg);
        parent::getMessage();
    }
}

class midgard_setup_ui_cli extends midgard_setup_ui
{
    private $database_create;
    private $database_update;
    public $midgard_config;
    public $exception;

    public function __construct()
    {
        parent::__construct();

        $this->database_create = FALSE;
        $this->database_update = FALSE;

        /* Set midgard_config defaults */
        $this->midgard_config->dbtype = 'MySQL';
        $this->midgard_config->database = 'midgard';
        $this->midgard_config->dbuser = 'midgard';
        $password = substr(md5(rand()), 0, 10);
        $this->midgard_config->dbpass = &$password;
        $this->midgard_config->host = 'localhost';

	    midgard_setup_log::write(_("Initialize new ").__CLASS__);
    }

    public function get_database_name()
    {
        $value = parent::get_text(_("What is the name for Midgard database?")); 

        if ($value == "") 
        {
            parent::warning(_("Database name can not be empty"));
            $this->get_database_name();
        } 

        $this->midgard_config->database = $value;
    }

    public function get_database_username()
    {    
        $value = parent::get_text(_("What is the username for Midgard database?")); 

        if($value == "") 
        {            
            parent::warning(_("Username can not be empty"));
            $this->get_database_username();
        } 

        $this->midgard_config->dbuser = $value;
    }

    public function get_database_password()
    {
        $value = parent::get_text(_("What is the password for Midgard database?")); 

        if($value == "") 
        {            
            parent::warning(_("Password can not be empty"));
            $this->get_database_password();
        } 

        $this->midgard_config->dbpass = $value; 
    }

    public function get_database_host()
    {
        $value = 
            parent::get_text(_("Host address for Midgard database? ( localhost by default)")); 

        if($value == "") 
        {
            $this->midgard_config->dbhost = 'localhost';
            return;
        }        
       
        $this->midgard_config->host = $value; 
    }

    public function get_database_create()
    {
        $value = parent::get_boolean(_("Create Midgard database?"));

        if($value == 'y') 
        {
            $this->database_create = TRUE;
            
            /* Note that midgard_config tablecreate switch doesn't describe
             * if database itself should be created if doesn't exists.
             * But we can use and perform database creation "in the background".
             */
            $this->midgard_config->tablecreate = TRUE;
        }
    }    

    public function get_database_update()
    {
        $value = parent::get_boolean(_("Update Midgard database?"));

        if($value == 'y') 
        {
            $this->database_update = TRUE;
            $this->midgard_config->tableupdate = TRUE;
        }
    }

    public function get_blobdir()
    {
        if($value == "") 
        {            
            parent::message(_("Blobdir set to default location"));
        } 

        /* TODO, validate blobdir location*/

        $this->midgard_config->blobdir = $value;       
    }

    public function save_config($name = NULL, $user = FALSE)
    {
        midgard_setup_log::write(_("Saving configuration:").$name);

        if(!is_object($this->midgard_config)) 
        {		
            parent::error(_("midgard_config not associated"));
            return FALSE;
        }

        if($name == NULL)
            $name = "midgard";
            
        $this->midgard_config->save_file($name, $user);
    }

    static public function get_text($question, $default = "")
    {
        fwrite(STDOUT, " \n{$question} \n({$default}): ");
        
        $value = fgets(STDIN);
        
        return trim($value);
    }

    static public function get_boolean($question, $default = "y")
    {
        fwrite(STDOUT, "{$question}\n({$default}): ");
        fwrite(STDOUT, "\t y: Yes \n");
        fwrite(STDOUT, "\t n: No \n");
        fwrite(STDOUT, "\t q: Quit \n");
        
        do {

            do {
            
                $answer = fgets(STDIN);
            
            } while ( trim($answer) == '' );
            
            $selected = trim(strtolower($answer));
        
            if($selected == 'n' || $selected == 'y') {
        
                return $selected;
        
            } else {
            
                if($selected == 'q')
                    return NULL;
                
                fwrite(STDOUT, "Please, select 'y' or 'n' \n");
            }

        } while($answer != 'q');
    }
    
    static public function get_password($question)
    {
        fwrite(STDOUT, "This is not secure!");
        fwrite(STDOUT, "{$question}\n");
         
        $value = fgets(STDIN);
       
        return $value;
    }

    static public function monty()
    {
        fwrite(STDOUT, "What Is Your Favourite Colour?\n");
        sleep(1);
        fwrite(STDOUT, "Red");
        sleep(1);
        fwrite(STDOUT, "...No! No! Blue! \n");
        sleep(1);
        fwrite(STDOUT, "a\n a\n  a\n   a\n    a\n     a\n     \n");
        exit;
    }

    static public function message($text)
    {
    	midgard_setup_log::write($text);
        fwrite(STDOUT, "MIDGARD SETUP: $text \n");
    }

    static public function warning($text)
    {
    	midgard_setup_log::write("\n ! WARNING ! {$text}");
        fwrite(STDOUT, "\nMIDGARD SETUP: ! WARNING ! : {$text} \n");
    }

    static public function error($text)
    {
    	midgard_setup_log::write("ERROR {$text} \n SETUP FAILED");
        fwrite(STDOUT, "\nMIDGARD SETUP ERROR: {$text} \nQuitting...\n");
        exit(1);
    }
}
?>
